/*
   *Copyright:  

   ACEMod (2018--2021), The University of Sydney's invention disclosure CDIP Ref. 2019-123.
   Copyright   Oliver Cliff, Cameron Zachreson, Nathan Harding, Kristopher Fair, Mikhail Prokopenko, Mahendarajah Piraveenan

   AMTraC-19 (2020-201), The University of Sydney's invention disclosure CDIP 2020-018 (2020).
   Copyright   Sheryl Chang, Nathan Harding, Cameron Zachreson, Oliver Cliff, Mikhail Prokopenko

   This code is released only for internal review by Nature Communications, manuscript: NCOMMS-21-29568.

   *EndCopyright:
*/

#include "agent.h"
#include "debug.h"
#include "enums.h"

#include <iostream>
#include <cstdlib>
#include <math.h>
#include <algorithm>

#define P_CHILD    0
#define P_ADULT    1

extern double WDpro[3][3];

//constexpr double   Agent::PERCENT_SYMPTOMATIC;
//constexpr double   Agent::CHILD_PERCENT_SYMPTOMATIC;
//constexpr unsigned Agent::INFECTIOUS_PERIOD_;
//constexpr unsigned Agent::PEAK_VIRAL_TITER_;

constexpr float Agent::DAYS_PER_STEP_;


double Agent::computeViralLoad(int t)
{
   double viral_titer;

   if (t < this->getLATENT_PERIOD_())
    {
       viral_titer = 0;
    }

   else if (t < this->getPEAK_VIRAL_TITER_())
    {
       double x = double( t - this->getLATENT_PERIOD_()) / double( this->getPEAK_VIRAL_TITER_() - this->getLATENT_PERIOD_());
       viral_titer = (pow(Agent::exponent_, x) - 1) / (Agent::exponent_ - 1);
    }

   else
    {
       viral_titer = double( this->getINFECTION_PERIOD_() - t ) / double( this->getINFECTION_PERIOD_() - this->getPEAK_VIRAL_TITER_());
       //viral_titer = double( this->getINFECTIOUS_PERIOD_() - t ) / double( this->getINFECTIOUS_PERIOD_() - this->getPEAK_VIRAL_TITER_());
       // double x = double (t- Agent:: PEAK_VIRAL_TITER_ ) /double(Agent::INFECTIOUS_PERIOD_-Agent::PEAK_VIRAL_TITER_);
       // viral_titer = pow(Agent::exponent2_,-x);
    }

   return(viral_titer);
}


void Agent::printTiterVsTime(void)

{
   std::vector<double> viral_load_;

   for (unsigned t = 0; t < this->getLATENT_PERIOD_(); ++t)
    {
       viral_load_.push_back(0);
    }

   double viral_titer;
   for (unsigned t = this->getLATENT_PERIOD_(); t <= this->getINFECTION_PERIOD_(); ++t)
    {
       if (t < this->getPEAK_VIRAL_TITER_())
        {
           double x = double( t - this->getLATENT_PERIOD_()) / double( this->getPEAK_VIRAL_TITER_() - this->getLATENT_PERIOD_());
           viral_titer = (pow(Agent::exponent_, x) - 1) / (Agent::exponent_ - 1);
        }
       else
        {
           viral_titer = double( this->getINFECTION_PERIOD_() - t ) / double( this->getINFECTION_PERIOD_() - this->getPEAK_VIRAL_TITER_());
           // double x = double (t- Agent:: PEAK_VIRAL_TITER_ ) /double(Agent::INFECTIOUS_PERIOD_-Agent::PEAK_VIRAL_TITER_);
           // viral_titer = pow(Agent::exponent2_,-x);
        }
       viral_load_.push_back(viral_titer);
    }


   for (unsigned i = 0; i < viral_load_.size(); ++i)
    {
       std::cout << "titer[" << i << "]: " << viral_load_.at(i) << std::endl;
    }
}


/*
 * void Agent::computeViralLoad(bool print_titer, int t)
 * {
 * viral_load_.clear();
 *
 * for (unsigned t = 0; t < Agent::LATENT_PERIOD_; ++t)
 * {
 *    viral_load_.push_back(0);
 * }
 *
 * double viral_titer;
 * for (unsigned t = Agent::LATENT_PERIOD_; t <= Agent::INFECTIOUS_PERIOD_; ++t)
 * {
 *    if (t < Agent::PEAK_VIRAL_TITER_)
 *    {
 *       double x = double( t - Agent::LATENT_PERIOD_ ) / double( Agent::PEAK_VIRAL_TITER_ - Agent::LATENT_PERIOD_ );
 *       viral_titer = (pow(Agent::exponent_, x) - 1) / (Agent::exponent_ - 1);
 *    }
 *    else
 *    {
 *       viral_titer = double( Agent::INFECTIOUS_PERIOD_ - t ) / double( Agent::INFECTIOUS_PERIOD_ - Agent::PEAK_VIRAL_TITER_ );
 *       // double x = double (t- Agent:: PEAK_VIRAL_TITER_ ) /double(Agent::INFECTIOUS_PERIOD_-Agent::PEAK_VIRAL_TITER_);
 *       // viral_titer = pow(Agent::exponent2_,-x);
 *    }
 *    viral_load_.push_back(viral_titer);
 * }
 *
 * if (print_titer)
 * {
 *    for (unsigned i = 0; i < viral_load_.size(); ++i)
 *    {
 *       std::cout << "titer[" << i << "]: " << viral_load_.at(i) << std::endl;
 *    }
 * }
 * }
 */
void Agent::init(void)
{
   /*
    * Either symptomatic (with given incubation period)
    * Or asymptomatic (with incubation period -1)
    */
   double symp_rval = 0.0;

   symp_rval = rand() / (double)RAND_MAX;

   double symptomatic_fraction = PERCENT_SYMPTOMATIC;

   if (!this->isAdult())
    {
       symptomatic_fraction = CHILD_PERCENT_SYMPTOMATIC;
    }

   if (symp_rval < symptomatic_fraction)
    {
       double inc_rval = rand() / (double)RAND_MAX;

       //incubating_period_ = this->getLATENT_PERIOD_();
       incubating_period_ = this->getPEAK_VIRAL_TITER_();

       /*if ( inc_rval < 0.3){
        * incubating_period_  = LATENT_PERIOD_;
        * }
        * else if ( inc_rval < 0.8)
        * {
        * incubating_period_ = 4;
        * }
        * else
        * {
        * incubating_period_ = 6;
        * } */
    }
   else
    {
       incubating_period_ = -1;
    }

   /* Since we add a day for latent period. */
   //incubating_period_ += LATENT_PERIOD_;

   state_                   = DiseaseStates::StateSusceptible;
   infected_prior_          = false;
   new_illness_             = false;
   newly_detected_          = false; // CZ: new label to differentiate detections from illnesses
   illness_detected_        = false; // resets illness detection flag, this resets CI status too ( or at least it should )
   detected_recovered_      = false;
   AVEd_illness_supression_ = incubating_period_;

   SDstate_                    = SD_States::SD_noncompliant; // CZ COV19 25032020 - resets SD state to noncomplian upon initiation (called at start of run)
   CIstate_                    = CI_States::CI_noncompliant; // CZ COV19 25032020 - resets SD state to noncomplian upon initiation (called at start of run)
   HQstate_                    = HQ_States::HQ_noncompliant;
   SCstate_                    = SC_States::SC_nil;
   Compliance_state_           = Compliance_States::nil;
   Isolation_state_            = Isolation_States::nil;
   isolation_factor_home_      = 1;
   isolation_factor_work_      = 1;
   isolation_factor_community_ = 1;
   HQclock_                    = 0;

   steps_since_infection_ = 0;
}


void Agent::step(void)
{
   if ((this->getState() > DiseaseStates::StateSusceptible) && (this->getState() < DiseaseStates::StateRemoved))
    {
       this->steps_since_infection_++;

       /*
        * If this agent isn't showing symptoms but should at some point (i.e., inc period > 0),
        * check if should start showing symptoms
        */
       if ((this->getState() == DiseaseStates::StateAsymptomatic) &&
           (this->incubating_period_ > 0) &&
           (this->steps_since_infection_ > incubating_period_))
        {
           //std::cout << "Agent " << this->getID() << " state change: asymptomatic->symptomatic (" << this->steps_since_infection_ << " steps since infection)." << std::endl;
           this->setState(DiseaseStates::StateSymptomatic);
           this->new_illness_ = true;
        }



       // CZ COV19 31032019 if agent is detected, make sure households are updated accordingly, as their status is re-zeroed each round
       if (this->isIllnessDetected())
        {
           this->getHH()->addSymptomaticAgent();
        }


       /*
        * If infected for 7 days remove agent
        */

       // subtract a day from the recovery time if the agent is using antivirals - Kit edit -intervention
       unsigned Recovery_steps = this->getINFECTION_PERIOD_();
       if ((this->getAVState() == AVStates::AV_prophylaxis) || (this->getAVState() == AVStates::AV_treatment))
        {
           Recovery_steps = Recovery_steps - 2;
        }                                                                                                                                      // AVE reduces recovery time by a day, a day is two steps.

       if (this->steps_since_infection_ > Recovery_steps)
        {
           //std::cout << "Agent " << this->getID() << " state change: infectious->removed (" << this->steps_since_infection_ << " steps since infection)." << std::endl;
           this->setState(DiseaseStates::StateRemoved);

           if (this->illness_detected_)
            {
               this->detected_recovered_ = true; // precludes vaccination
            }

           this->illness_detected_ = false; // CZ COV19 - making sure detected individuals are counted correctly

           this->getCD()->remove_n_infecteds_tot();
           if (this->getCD()->get_n_infecteds_tot() == 0)
            {
               this->getCD()->uninfect_CD();
            }
        }
    }
}


// CZ COV19 31032020
void Agent::step_HQ(int duration_HQ)
{
   if (this->getIsolationState() == Isolation_States::HQ)
    {
       this->HQclock_ += 1;
    }

   if (this->HQclock_ > duration_HQ)
    {
       this->setIsolationState(Isolation_States::nil);
       this->HQclock_ = 0;
    }
}


// CZ COV19 - which of the 6 compliance categories is the Agent in?
// note: school closure compliance is not included here.
void Agent::eval_Compliance_state(void)
{
   if ((this->getCIState() == CI_States::CI_compliant) &&
       (this->getHQState() == HQ_State::HQ_compliant) &&
       (this->getSDState() == SD_State::SD_compliant))
    {
       this->setComplianceState(Compliance_States::SD_CI_HQ);
    }

   else if ((this->getCIState() == CI_States::CI_compliant) &&
            (this->getHQState() == HQ_State::HQ_compliant) &&
            (this->getSDState() == SD_State::SD_noncompliant))
    {
       this->setComplianceState(Compliance_States::CI_HQ);
    }

   else if ((this->getCIState() == CI_States::CI_compliant) &&
            (this->getHQState() == HQ_State::HQ_noncompliant) &&
            (this->getSDState() == SD_State::SD_compliant))
    {
       this->setComplianceState(Compliance_States::SD_CI);
    }

   else if ((this->getCIState() == CI_States::CI_compliant) &&
            (this->getHQState() == HQ_State::HQ_noncompliant) &&
            (this->getSDState() == SD_State::SD_noncompliant))
    {
       this->setComplianceState(Compliance_States::CI);
    }

   else if ((this->getCIState() == CI_States::CI_noncompliant) &&
            (this->getHQState() == HQ_State::HQ_noncompliant) &&
            (this->getSDState() == SD_State::SD_compliant))
    {
       this->setComplianceState(Compliance_States::SD);
    }

   else if ((this->getCIState() == CI_States::CI_noncompliant) &&
            (this->getHQState() == HQ_State::HQ_noncompliant) &&
            (this->getSDState() == SD_State::SD_noncompliant))
    {
       this->setComplianceState(Compliance_States::nil);
    }
}


int Agent::findParent_ID(agent_map_type& agents)
{
   int parent_ID = -2;
   int agent_ID;

   if (!this->isStudent())
    {
       std::cout << "WARNING: trying to find parent of non-student" << std::endl;
       return(-1);
    }
   else
    {
       std::vector<unsigned long> IDs_HH = this->getHH()->getAgentIdxs();

       std::random_shuffle(IDs_HH.begin(), IDs_HH.end());

       for (int i = 0; i < IDs_HH.size(); i++)
        {
           agent_ID = this->getHH()->getAgentIdxs().at(i);

           if (agents.at(agent_ID)->isAdult())
            {
               parent_ID = agent_ID;

               if ((agents.at(parent_ID)->getSCState() == SC_States::SC_parent) ||
                   (agents.at(parent_ID)->getSCState() == SC_States::SC_teacher))
                {
                   return(parent_ID); // if a parent is already staying home, just choose them
                }
            }
        }
    }

   if (parent_ID < 0)
    {
       std::cout << "WARNING: unable to find parent in student's household" << std::endl;
    }
   return(parent_ID);
}


//CZ COV19 01042020, from isolation states, compute isolation-related FoI reduction factors

void Agent::setIsolationFactor_home(double FoI_SD_home, double FoI_CI_home, double FoI_HQ_home, double FoI_SC_home)
{
   if (this->getIsolationState() == Isolation_States::CI)
    {
       this->isolation_factor_home_ = FoI_CI_home;
    }
   else if (this->getIsolationState() == Isolation_States::HQ)
    {
       this->isolation_factor_home_ = FoI_HQ_home;
    }
   else if (this->getIsolationState() == Isolation_States::SD)
    {
       this->isolation_factor_home_ = FoI_SD_home;
    }
   else if (this->getIsolationState() == Isolation_States::SC)
    {
       this->isolation_factor_home_ = FoI_SC_home;
    }
   else
    {
       this->isolation_factor_home_ = 1;
    }
}


void Agent::setIsolationFactor_work(double FoI_SD_work, double FoI_CI_work, double FoI_HQ_work, double FoI_SC_work)
{
   if (this->getIsolationState() == Isolation_States::CI)
    {
       this->isolation_factor_work_ = FoI_CI_work;
    }
   else if (this->getIsolationState() == Isolation_States::HQ)
    {
       this->isolation_factor_work_ = FoI_HQ_work;
    }
   else if (this->getIsolationState() == Isolation_States::SD)
    {
       this->isolation_factor_work_ = FoI_SD_work;
    }
   else if (this->getIsolationState() == Isolation_States::SC)
    {
       this->isolation_factor_work_ = FoI_SC_work;
    }
   else
    {
       this->isolation_factor_work_ = 1;
    }
}


void Agent::setIsolationFactor_community(double FoI_SD_community, double FoI_CI_community, double FoI_HQ_community, double FoI_SC_community)
{
   if (this->getIsolationState() == Isolation_States::CI)
    {
       this->isolation_factor_community_ = FoI_CI_community;
    }
   else if (this->getIsolationState() == Isolation_States::HQ)
    {
       this->isolation_factor_community_ = FoI_HQ_community;
    }
   else if (this->getIsolationState() == Isolation_States::SD)
    {
       this->isolation_factor_community_ = FoI_SD_community;
    }
   else if (this->getIsolationState() == Isolation_States::SC)
    {
       this->isolation_factor_community_ = FoI_SC_community;
    }
   else
    {
       this->isolation_factor_community_ = 1;
    }
}


// This function checks if the symptomatic agents are detected.
//If they are it adds them to the symptomatic pool of infection,
//makes them recieve AV in 1 day and gives them a AVEd chance of being asymptomatic
bool Agent::Illness_detection(double& Detection, int Response_delay, double& TAP, double& GTAP)
{
   bool   agent_detected = false;
   double detectP        = 0.0;

   // If the sick agent is detected then add them to the symptomatic area
   detectP = rand() / (double)RAND_MAX;
   if (detectP < Detection)
    {
       agent_detected = true;

       this->illness_detected_ = true;

       // CZ: new illness flag is now updated upon detection, not symptom onset
       // this means that incidence curves reported will correspond to
       // detected cases - removing this from the step() function.
       this->newly_detected_ = true;

       // Adding symptomatic agents to the detection pools
       this->getHH()->addSymptomaticAgent(); // may add index case
       this->getCD()->addSymptomaticAgent();
       for (auto c_wg : working_groups_)
        {
           c_wg->addSymptomaticAgent();
        }

       if ((TAP > 0) || (GTAP > 0)) // Additional effects for AV
        {
           this->setAVState(AVStates::ExposedAV);
           this->steps_since_av_exposure_ = Response_delay - 1;
           // Set the AV exposure clock so detected agents will be treated in 1 day.
        }
    }
   return(agent_detected);
}




void Agent::addToNighttimeProbabilityOfInfection(double& VEi_1, double& VEi_2, double& AVEi)   /* Kit's edit -intervention*/
{
   //std::cout << "computing factor" << std::endl;
   double factor = this->computeFactor(VEi_1, VEi_2, AVEi);

   // The factor added to the force of infection is subjected to the multiplier    Kit's edit -AVE
//std::cout << "adding to pHH" << std::endl;
   this->addToHouseholdProbability(factor);
   // std::cout << "adding to pHC" << std::endl;
   this->addToClusterProbability(factor);
   //std::cout << "adding to pCom" << std::endl;
   this->addToCommunityProbability(factor);
   //std::cout << "adding to pNH" << std::endl;
   this->addToNeighbourhoodProbability(factor);
}


// CZ COV19 SD 17032020
void Agent::addToNighttimeProbabilityOfInfection_SD(double& VEi_1, double& VEi_2, double& AVEi)
{
   double factor = this->computeFactor(VEi_1, VEi_2, AVEi);

   this->addToHouseholdProbability(factor * this->getIsolationFactor_home());
   this->addToClusterProbability(factor * this->getIsolationFactor_community());
   this->addToCommunityProbability(factor * this->getIsolationFactor_community());
   this->addToNeighbourhoodProbability(factor * this->getIsolationFactor_community());
}


double Agent::getNighttimeProbabilityOfInfection(void)
{
   /*
    * Get the joint probability of infection, given where the agent interacts.
    *
    * Household: 5 element vector specifying Agegroup (5 groups)
    * Household Cluster: 2 element vector specifying child or adult (2 groups)
    * Census District: 5 element vector of age groups (5 groups)
    * Statistical local area: 5 element vector of age groups (5 groups)
    */
   return(1.0 - this->getHH()->getProbabilityOfTransmission()
          * this->getHC()->getProbabilityOfTransmission(this->isAdult())
          * this->getCD()->getProbabilityOfTransmission()
          * this->getSLA()->getProbabilityOfTransmission());
}


double Agent::getHouseholdProbabilityOfInfection(void)
{
   return(1.0 - this->getHH()->getProbabilityOfTransmission());
}


double Agent::getHouseholdClusterProbabilityOfInfection(void)
{
   return(1.0 - this->getHC()->getProbabilityOfTransmission(this->isAdult()));
}


double Agent::getNeighbourhoodProbabilityOfInfection(void)
{
   double P_not_trans = this->getCD()->getProbabilityOfTransmission();

   if (P_not_trans < 1 - 1e-4)
    {
       //std::cout << "Agent " << this->getID() << " 1-p(neighbourhood infection): " << P_not_trans << std::endl;
    }

   return(1.0 - P_not_trans);
}


double Agent::getCommunityProbabilityOfInfection(void)
{
   double P_not_trans = this->getSLA()->getProbabilityOfTransmission();

   if (P_not_trans < 1 - 1e-4)
    {
       //std::cout << "Agent " << this->getID() << " 1-p(community infection): " << P_not_trans << std::endl;
    }

   return(1.0 - P_not_trans);
}


void Agent::addToDaytimeProbabilityOfInfection(double& VEi_1, double& VEi_2, double& AVEi)
{
   double factor = this->computeFactor(VEi_1, VEi_2, AVEi);

   for (auto c_wg : working_groups_)
    {
       c_wg->addInfectedAgent(factor);
    }
}


//CZ COV19 SD 17032020
void Agent::addToDaytimeProbabilityOfInfection_SD(double& VEi_1, double& VEi_2, double& AVEi)
{
   double factor = this->computeFactor(VEi_1, VEi_2, AVEi);

   {
      for (auto c_wg : working_groups_)
       {
          //c_wg->addInfectedAgent(factor * 0.25); // assuming this reduces nonhousehold contacts by 75% for Case Isolation
          c_wg->addInfectedAgent(factor * this->getIsolationFactor_work());   // CZ COV19 24032020 - changing this. Case isolation means no workplace contacts, period. Home and community may still be relevant, however.
       }
   }
}


double Agent::getDaytimeProbabilityOfInfection(void)
{
   double P_not_infection_prod = 1.0;

   for (auto c_wg : working_groups_)
    {
       P_not_infection_prod *= c_wg->getProbabilityOfTransmission();
    }

   return(1.0 - P_not_infection_prod);
}


double Agent::getDaytimeProbabilityOfInfection(unsigned type)
{
   if (working_groups_.size() < type)
    {
       std::cerr << "Working group vector length: " << working_groups_.size() << " (requested " << type << ")" << std::endl;
    }
   return(1.0 - working_groups_.at(type)->getProbabilityOfTransmission());
}


void Agent::infect(double& AVEd, double& VEd_1, double& VEd_2)
{
   this->getCD()->add_n_infecteds_tot();

   // if agent's CD is uninfected, increase newly infected CD counter
   if (this->getCD()->get_n_infecteds_tot() == 1)
    {
       this->getCD()->infect_CD(); // CD has at least one newly infected agent
    }

   // This function is used to infect an individal;
   this->setState(DiseaseStates::StateAsymptomatic);
   this->steps_since_infection_ = 0;
   this->infected_prior_        = true;

   //KF CZ This has been moved from the init function
   double symp_rval = 0.0;
   symp_rval = rand() / (double)RAND_MAX;

   double symptomatic_fraction = this->getPERCENT_SYMPTOMATIC();

   if (!this->isAdult())
    {
       symptomatic_fraction = this->getCHILD_PERCENT_SYMPTOMATIC();
    }

   if (this->getAVState() == AVStates::AV_prophylaxis)
    {
       symptomatic_fraction *= (1 - AVEd);
    }

// CZCOVID-19 v7.5 adding vaccine efficacy for disease
   if (this->getVacState() == VacStates::vaccinated_general)
    {
       symptomatic_fraction *= (1 - VEd_2);
    }
   else if (this->getVacState() == VacStates::vaccinated_priority)
    {
       symptomatic_fraction *= (1 - VEd_1);
    }

   if (symp_rval < symptomatic_fraction)
    {
       double inc_rval = rand() / (double)RAND_MAX;

       //incubating_period_ = this->getLATENT_PERIOD_();
       incubating_period_ = this->getPEAK_VIRAL_TITER_();
    }
   else
    {
       incubating_period_ = -1;
    }
}


#if 1
double Agent::computeFactor(double& VEi_1, double&VEi_2, double& AVEi)
{
   if ((this->getState() > DiseaseStates::StateSusceptible) && (this->getState() < DiseaseStates::StateRemoved))
    {
       // 28.02.19 begin{}
       double vac_multiplier  = 1;
       double AVEi_multiplier = 1;
       if (this->getVacState() == VacStates::vaccinated_general)
        {
           vac_multiplier = 1 - VEi_2;
        }
       else if (this->getVacState() == VacStates::vaccinated_priority)
        {
           vac_multiplier = 1 - VEi_1;
        }                                                                                                      // This is the intervention multiplier for VP for infectiousness.

       if ((this->getAVState() == AVStates::AV_prophylaxis) || (this->getAVState() == AVStates::AV_treatment)) // If the agent has AV
        {
           // This is the intervention multiplier for AVP for infectiousness.
           AVEi_multiplier = 1 - AVEi;

           // Ensuring that the AV makes some additional infected agents (determined by AVEd) to behave asymptomaticly
        }


       //double factor = viral_load_.at(this->steps_since_infection_ - 1); //If the incubating period is -1 the agent stays asymptomatic

       double factor = computeViralLoad(this->steps_since_infection_ - 1);

       /*if (this->getState() == DiseaseStates::StateAsymptomatic)         // If agent is asymptomatic or if they are made asymptomatic by the AV
        * {
        * factor *= Agent::ASYMPTOMATIC_INFECTIVITY_;
        * }*/

       // CZ 2020 11 27 - adjusting this function so that asymptomatic infectiity does not apply to those who are pre-symptomatic
       // this is a bit hacky - but those who are pre-symptomatic have an incubating period > 0, while those who are
       // asymptomatic (will never have symptoms) have an incubating period of -1

// If agent is asymptomatic or if they are made asymptomatic by the AV
       if (this->getState() == DiseaseStates::StateAsymptomatic && this->incubating_period_ < 0)
        {
           factor *= Agent::ASYMPTOMATIC_INFECTIVITY_;
        }


       factor *= vac_multiplier * AVEi_multiplier; // Reducing the infectiousness factor by the vaccination and AV multipliers
       // 28.02.19 end{}

       return(factor);
    }

   return(0.0);
}


/*
 * double Agent::computeFactor(void)
 * {
 * if ((this->getState() > DiseaseStates::StateSusceptible) && (this->getState() < DiseaseStates::StateRemoved))
 * {
 *                                                                                                   // This is the intervention multiplier for VP for infectiousness.
 *    double factor = viral_load_.at(this->steps_since_infection_ - 1); //If the incubating period is -1 the agent stays asymptomatic
 *
 *    if (this->getState() == DiseaseStates::StateAsymptomatic)         // If agent is asymptomatic or if they are made asymptomatic by the AV
 *    {
 *       factor *= Agent::ASYMPTOMATIC_INFECTIVITY_;
 *    }
 *
 *    return(factor);
 * }
 *
 * return(0.0);
 * }
 */

#else
double Agent::computeFactor(void)
{
   if ((this->getState() > DiseaseStates::StateSusceptible) && (this->getState() < DiseaseStates::StateRemoved))
    {
       double factor = 0.0;
       if (this->steps_since_infection_ > LATENT_PERIOD_)
        {
           factor = 0.5;
           if (this->getState() == DiseaseStates::StateSymptomatic)
            {
               factor = viral_load_.at(this->steps_since_infection_ - 1);
            }
        }

       if (this->willBeSymptomatic())
        {
           std::cout << "Symptomatic agent " << this->getID() << " (current state " << this->getState() << ") factor for step " << this->steps_since_infection_ << ": " << factor << std::endl;
        }
       else
        {
           std::cout << "Asymptomatic agent " << this->getID() << " (current state " << this->getState() << ") factor for step " << this->steps_since_infection_ << ": " << factor << std::endl;
        }

       return(factor);
    }

   return(0.0);
}


#endif

/* beginning Kit's edit - intervention*/
// Stepping the AV count since exposure to apply the AVE and count the additional doses every 10 days after the intial dose.
int Agent::stepAV(int& Response_delay)   //
{
   int new_dose = 0;

   // Count the days since exposure
   this->steps_since_av_exposure_++;

   // Count the days since AV applied
   if ((this->getAVState() == AVStates::AV_prophylaxis) || (this->getAVState() == AVStates::AV_treatment))
    {
       this->steps_since_av_applied_++;
    }

   // if steps since exposure is greater than the intervention delay then the agent status is changed so they recieve AVP
   if (this->steps_since_av_exposure_ == (Response_delay + 1))
    {
       if (this->getState() == DiseaseStates::StateSymptomatic) // If the agent is infected their AV will only last 5 days
        {
           this->setAVState(AVStates::AV_treatment);            //AV_treatment
        }
       else // If the agent isn't infected the AV will last 10 days
        {
           this->setAVState(AVStates::AV_prophylaxis); //AV_prophylaxis
        }

       this->steps_since_av_applied_ = 0; // Start the days since AV applied count
       new_dose                      = 1;
    }



   if ((this->getAVState() == AVStates::AV_prophylaxis) && (this->steps_since_av_applied_ == 11))
   // Non-infected agent becomes susceptible again after the course runs out
    {
       this->setAVState(AVStates::Notexposed);
       this->steps_since_av_exposure_ = 0;
       this->incubating_period_       = this->AVEd_illness_supression_;
       // Changing back to the previous distribution of asymptomatic
    }
   else if ((this->getAVState() == AVStates::AV_treatment) && (this->steps_since_av_applied_ == 6))
   // Infected agent because susceptible again after the course runs out
    {
       if (this->isIllnessDetected())                                                               // Has the agent also been detected? if so they will not be treated again
        {
           this->setAVState(AVStates::DetectedTreated);
        }
       else
        {
           this->setAVState(AVStates::Notexposed);
           this->steps_since_av_exposure_ = 0;
           this->incubating_period_       = this->AVEd_illness_supression_;
           // Changing back to the previous distribution of asymptomatic
        }
    }

   // Adds a dose to the count if the agent recieved AV
   return(new_dose);
}


// Identifying HH and WG regions that have symptomatic infection in them
bool Agent::TAP_Sympotmatic_present(void)
{
   bool Sympotmatic_infection = false;

   if (this->getHH()->getSymptomaticInfection())
    {
       Sympotmatic_infection = true;
    }
   else
    {
       for (auto c_wg : working_groups_)
        {
           if (c_wg->getSymptomaticInfection())
            {
               Sympotmatic_infection = true;
            }
        }
    }
   return(Sympotmatic_infection);
}


// Identifying SA1 regions that have symptomatic infection in them
bool Agent::CD_Sympotmatic_present(void)
{
   bool Sympotmatic_infection = false;

   if (this->getCD()->getSymptomaticInfection())
    {
       Sympotmatic_infection = true;
    }
   return(Sympotmatic_infection);
}


// Start quarantine day count untill the agent is quarantined
void Agent::reset_intervention(void)
{
   this->steps_since_av_exposure_ = 0;
   this->steps_since_av_applied_  = 0;
}


// Start the GTAP day count untill the agent is given AV
void Agent::GTAP_administer(double& GTAP)
{
   if (rand() / (double)RAND_MAX < GTAP)
    {
       this->setAVState(AVStates::ExposedAV);
       this->steps_since_av_exposure_ = 0;
    }
   else
    {
       this->setAVState(AVStates::ExposedNoAV);
    }
}


// Start the TAP day count untill the agent is given AV
void Agent::TAP_administer(double& TAP)
{
   if (rand() / (double)RAND_MAX < TAP)
    {
       this->setAVState(AVStates::ExposedAV);
       this->steps_since_av_exposure_ = 0;
    }
   else
    {
       this->setAVState(AVStates::ExposedNoAV);
    }
}
