/*
   *Copyright:  

   ACEMod (2018--2021), The University of Sydney's invention disclosure CDIP Ref. 2019-123.
   Copyright   Oliver Cliff, Cameron Zachreson, Nathan Harding, Kristopher Fair, Mikhail Prokopenko, Mahendarajah Piraveenan

   AMTraC-19 (2020-201), The University of Sydney's invention disclosure CDIP 2020-018 (2020).
   Copyright   Sheryl Chang, Nathan Harding, Cameron Zachreson, Oliver Cliff, Mikhail Prokopenko

   This code is released only for internal review by Nature Communications, manuscript: NCOMMS-21-29568.

   *EndCopyright:
*/

#ifndef EPIDEMIC_H_
#define EPIDEMIC_H_

#include "agent.h"
#include "transmission_rates.h"
#include "statistical_areas.h"
#include "debug.h"

#include <cstdlib>
#include <iostream>
#include <stdarg.h>

class Epidemic
{
private:

   /* List of potential seed SLAs (near intl. airports) and how many people fly to these airports */
   static std::vector <std::pair <unsigned long, std::vector <unsigned long> > > n_commuters_to_airport_slas_;

   static std::vector <std::pair <unsigned long, std::vector <unsigned long> > > n_commuters_to_airport_agents_;

   static std::vector <unsigned long> agent_list_;

   static std::string seeded_sla_fname;

   const static double expected_number_infected_, transmission_scaler_;

public:

   static void init_transmission_rates(double, double);

   static void init_airport_seeded_slas(const std::string&, const sla_map_type&);

   static void init_random_population_seed(const agent_map_type&);

   static void seed_epi_sla(unsigned int, StatisticalLocalArea&, agent_map_type&, double& AVEd, double& VEd_1, double& VEd_2);

   static void seed_epi_agents(double, agent_map_type&, double& AVEd, double& VEd_1, double& VEd_2);

   static void seed_epi_airports(agent_map_type&, std::vector <unsigned long>&, double& AVEd, double& VEd_1, double& VEd_2);

   static unsigned get_one_airport_agent_idx(unsigned, agent_map_type&);
};

#endif // IC_EPI_H_
