/*
   *Copyright:  

   ACEMod (2018--2021), The University of Sydney's invention disclosure CDIP Ref. 2019-123.
   Copyright   Oliver Cliff, Cameron Zachreson, Nathan Harding, Kristopher Fair, Mikhail Prokopenko, Mahendarajah Piraveenan

   AMTraC-19 (2020-201), The University of Sydney's invention disclosure CDIP 2020-018 (2020).
   Copyright   Sheryl Chang, Nathan Harding, Cameron Zachreson, Oliver Cliff, Mikhail Prokopenko

   This code is released only for internal review by Nature Communications, manuscript: NCOMMS-21-29568.

   *EndCopyright:
*/

#ifndef GENERATE_POPULATION_H_
#define GENERATE_POPULATION_H_

#include "enums.h"

/*
 * Sample spaces for CDFs
 */
const std::vector <unsigned> hh_comps { /* Single/lone household */
   HouseholdCompositions::Single,
   /* Couples without children */
   HouseholdCompositions::CouplesWithoutChildren,
   /* Couples with children */
   HouseholdCompositions::CouplesWithOneChild,
   HouseholdCompositions::CouplesWithTwoChildren,
   HouseholdCompositions::CouplesWithThreeChildren,
   HouseholdCompositions::CouplesWithFourChildren,
   /* Single parent families */
   HouseholdCompositions::SingleParentFamilyOneChild,
   HouseholdCompositions::SingleParentFamilyTwoChildren,
   HouseholdCompositions::SingleParentFamilyThreeChildren,
   HouseholdCompositions::SingleParentFamilyFourChildren,
   HouseholdCompositions::SingleParentFamilyFiveChildren,
   /* Group (share) households */
   HouseholdCompositions::GroupTwo,
   HouseholdCompositions::GroupThree,
   HouseholdCompositions::GroupFour,
   HouseholdCompositions::GroupFive,
   HouseholdCompositions::GroupSix
};

const std::vector <Sex> sexes { Sexes::Male, Sexes::Female };

const std::vector <AgeGroup> adult_agegroups { AgeGroups::Age19to29, AgeGroups::Age30to64, AgeGroups::Age65Plus };

const std::vector <AgeGroup> child_agegroups { AgeGroups::Age0to4, AgeGroups::Age5to18 };
/* Useful typedefs for statistical areas */
typedef unsigned long long                                                       dz_type;
typedef unsigned long                                                            cd_type;
typedef unsigned long                                                            sla_type;
typedef unsigned long                                                            wg_type;

typedef std::map <cd_type, std::vector <std::pair <dz_type, unsigned long> > >   cd_dz_nagents_type;

/* Some constants for generating our population */

const double PERC_ADULTS_AT_HOME = 0.01;
//const double PERC_ADULTS_AT_HOME = 0.0;
const double SCHOOL_ATTENDANCE_LIKELIHOOD = 0.95;

const unsigned NUM_PEOPLE_IN_WG   = 20;
const unsigned HOUSES_PER_CLUSTER = 4;
//const double NUM_STUDENTS_TO_TEACHERS = 8.5;
const double    NUM_STUDENTS_TO_TEACHERS     = 8.5;
const double    SCHOOL_ZONE_SIGMA            = 0.25; //  (1 unit ~= 100km, i.e., average commute distance is 15km)
const double    MAX_STUDENT_SCALER           = 0.4;  // For SCHOOL-0.9.dat
const double    MAX_STUDENT_COMMUTE_DISTANCE = 1.5;  // Max travel distance is 150km
const unsigned MAX_ALLOCATION_ATTEMPTS      = 5;
const unsigned SCHOOL_STEP_SIZE             = 5;

const unsigned long DZ_MULTIPLIER  = 1000000000;
const unsigned long SLA_MULTIPLIER = 100000000;

const std::vector <std::string> STATE_STRINGS { "NSW", "VIC", "QLD", "SA", "WA", "TAS", "NT", "ACT" };
const std::vector <unsigned>    SCHOOL_BOUNDS { 0, 35, 100, 200, 300, 400, 600, 800, 1000, 2500 }; /* Max number of students in school */

/* Simple agent structure */
struct SimpleAgent
{
   unsigned long my_id;
   AgeGroup      my_age;
   cd_type       my_cd;
   signed long   my_wg;
   unsigned long my_hh;
   unsigned long my_hc;

   inline bool   isAdult(void)
   {
      return(!this->isChild());
   }

   inline bool   isChild(void)
   {
      return(my_age == AgeGroups::Age0to4 || my_age == AgeGroups::Age5to18);
   }

   inline bool   isStudentAge(void)
   {
      return(my_age == AgeGroups::Age5to18);
   }

   inline bool   isStudent(void)
   {
      return(my_wg < 0);
   }
};

struct WGLimitStruct
{
   unsigned long lower;
   unsigned long upper;
};

struct School
{
   unsigned long ID;
   unsigned long max_students;
   unsigned long num_students;
   unsigned long DZ;
   unsigned long SLA;
};

#endif // GENERATE_POPULATION_H_
