/*
   *Copyright:  

   ACEMod (2018--2021), The University of Sydney's invention disclosure CDIP Ref. 2019-123.
   Copyright   Oliver Cliff, Cameron Zachreson, Nathan Harding, Kristopher Fair, Mikhail Prokopenko, Mahendarajah Piraveenan

   AMTraC-19 (2020-201), The University of Sydney's invention disclosure CDIP 2020-018 (2020).
   Copyright   Sheryl Chang, Nathan Harding, Cameron Zachreson, Oliver Cliff, Mikhail Prokopenko

   This code is released only for internal review by Nature Communications, manuscript: NCOMMS-21-29568.

   *EndCopyright:
*/

#ifndef TRANSMISSION_RATES_H_
#define TRANSMISSION_RATES_H_

/*
 * CONTACT RATES (rescaled by p_trans)
*/

// These are taken from FluTe and the papers they used. The following are CONTACT RATES (all were originally xmit_ 
//Since they are contact rates, they will need to be rescaled

const std::vector< double > contact_rate_sla{ 0.0000109, 0.0000326, 0.000087, 0.000087, 0.000174 };

const std::vector< double > contact_rate_cd{ 0.0000435, 0.0001305, 0.000348, 0.000348, 0.000696 };

const double contact_rate_wg = 0.05;

/*
 * Household cluster rate matrix:
 * 	1x1: P_{CHILD->CHILD}
 * 	1x2: P_{CHILD->ADULT}
 * 	2x1: P_{ADULT->CHILD}
 * 	2x2: P_{ADULT->ADULT}
 */
const std::vector< std::vector< double > > contact_rate_hc{ std::vector< double >{ 0.05, 0.05 },
															std::vector< double >{ 0.05, 0.05 } };

/*
 * TRANSMISSION RATES (not rescaled)
*/

/* Whole school, grade, class  */
const std::vector< double > transmission_rate_schools{ 0.0011992803, 0.0064789207, 0.0349737732 };

/*
 * Each row repsents household size: 2, 3, 4, 5, 6+
 * Each column represents age group: 0-4, 5-18, 19-29, 30-64, 65+
 */

const std::vector< std::vector< double > > transmission_rate_hh{ std::vector< double >{ 0.3308952704, 0.3308952704, 0.0955729875, 0.0955729875, 0.0955729875 }, // 2
																 std::vector< double >{ 0.2188678988, 0.2188678988, 0.0598846984, 0.0598846984, 0.0598846984 },
																 std::vector< double >{ 0.1604590293, 0.1604590293, 0.042782844,  0.042782844,  0.042782844  },
																 std::vector< double >{ 0.1252462334, 0.1252462334, 0.0328998396, 0.0328998396, 0.0328998396 },
																 std::vector< double >{ 0.1019394474, 0.1019394474, 0.0265214084, 0.0265214084, 0.0265214084 } }; // 

#endif //TRANSMISSION_RATES_H_
