/*
   *Copyright:  

   ACEMod (2018--2021), The University of Sydney's invention disclosure CDIP Ref. 2019-123.
   Copyright ©  Oliver Cliff, Cameron Zachreson, Nathan Harding, Kristopher Fair, Mikhail Prokopenko, Mahendarajah Piraveenan

   AMTraC-19 (2020-201), The University of Sydney's invention disclosure CDIP 2020-018 (2020).
   Copyright ©  Sheryl Chang, Nathan Harding, Cameron Zachreson, Oliver Cliff, Mikhail Prokopenko

   This code is released only for internal review by Nature Communications, manuscript: NCOMMS-21-29568.

   *EndCopyright:
*/

#ifndef UTILS_HPP_
#define UTILS_HPP_

#include <map>
#include <algorithm>
#include <cmath>
#include <boost/math/special_functions/erf.hpp>

/* Flip the pair key<->value fields */
#if 0
template<typename A, typename B>
std::pair<B,A> flip_pair(const std::pair<A,B> &p)
{
    return std::pair<B,A>(p.second, p.first);
}

/* Flip the map key<->value fields (and sort by key?) */
template<typename A, typename B>
std::map<B,A> flip_map(const std::map<A,B> &src)
{
    std::map<B,A> dst;
    std::transform(src.begin(), src.end(), std::inserter(dst, dst.begin()), 
                   flip_pair<A,B>);
    return dst;
}
#endif

/* Return cumulative probability of a half-normal distribution at x (st dev sigma) */
inline float foldedNormCDF( float sigma, float x )
{
	return boost::math::erf( x / sqrt( 2 ) / sigma );
}

/* Return the value of the inverse CDF of a half-normal distribution at probability p (st dev sigma) */
inline float invFoldedNormCDF( float sigma, float p )
{
  p = ( p < 1e-4 ) ? 1e-4 : p;
  p = ( p > 0.999 ) ? 0.9999 : p;
	return sigma * sqrt( 2 ) * boost::math::erf_inv( p );
}

/*
 * Generate CDF from some unnormalised prob distribution (dist) and the bins
 */
template<class T>
std::vector< std::pair< T, float > > inline generateCDF( const std::vector< T > bins, const std::vector< float > dist )
{  
  std::vector< std::pair< T, float > > cdf;

  /* Cumulate distribution... */
  float sum = 0; 
  for ( unsigned i = 0; i < dist.size(); ++i )
  {
    sum += dist.at(i);
    cdf.emplace_back( bins.at(i), sum );
  }
   
  /* ...and normalise it */ 
  //std::cout << "CDF:\t";
  for ( auto& outcome : cdf )
  {
    outcome.second /= sum;
    //std::cout << "{" << outcome.first << "," << outcome.second << "}\t";
  }
  //std::cout << std::endl;
  return cdf;
}

template<class T>
T sampleCDF( const std::vector< std::pair< T, float > >& cdf )
{
  float rval = rand() / float( RAND_MAX );
  
  for ( const auto& outcome : cdf )
  {
    if ( rval <= outcome.second )
    {
      return outcome.first;
    }
  }
  return cdf.rbegin()->first;
}

#if 0
template<class T1, class T2>
static unsigned long int Find(T1 list[], T2 Value, unsigned long int N ) {

  unsigned long int low = 0;
  unsigned long int high = N;
  unsigned long int mid;

  while ( low <= high )
  {
    mid = low + ( high - low ) / 2;
    
    //std::cout << "Looking for " << Value << " at idx " << mid << "/" << N << " (value:" << list[mid].ID << ")" << std::endl;
    if ( list[mid].ID < Value )
    {
      low = mid + 1;
    }
    else if ( list[mid].ID > Value && mid != 0 )
    {
      high = mid - 1;
    }
    else
    {
      return mid;
    }
  }
  return -1;
}
#endif

#endif //UTILS_HPP_